"""This module contains commonly-used constants."""

from typing_extensions import Final

# The dictionary used for XRPL base58 encodings
XRPL_ALPHABET: Final[bytes] = (
    b"rpshnaf39wBUDNEGHJKLM4PQRST7VWXYZ2bcdeCg65jkm8oFqi1tuvAxyz"
)
